# -*- coding: utf-8 -*-


from enigma import eActionMap, eConsoleAppContainer, eEnv
from Components.ActionMap import ActionMap
from Components.ConfigList import ConfigListScreen, ConfigList
from Components.config import config, getConfigListEntry, ConfigSubsection, ConfigSelection, ConfigEnableDisable, ConfigYesNo, ConfigInteger, ConfigNumber, ConfigText, ConfigSubList
from Components.config import ConfigNothing, ConfigSubsectionContent, ConfigDescription
from Components.Console import Console
from Components.Label import Label
from Plugins.Plugin import PluginDescriptor
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox
from Tools.Notifications import AddNotification
from Tools.BoundFunction import boundFunction
from datetime import datetime
import time
from os import system, path, mkdir, makedirs
from shutil import copy
import os
from __init__ import _

pluginversion = "Release 1.02"

config.plugins.MyEPG = ConfigSubsection()
config.plugins.MyEPG.nothing = ConfigNothing()
config.plugins.MyEPG.EPG_version = ConfigSelection(default = "stay", choices = [("stay",  ("Aktuelle Version beibehalten")),  ("modified",  ("MyEPG installieren")), ("standard", ("Standard-EPG installieren"))])
config.plugins.MyEPG.activate_skins = ConfigYesNo(default = True)

config.plugins.MyEPG.enable_wrap_single = ConfigYesNo(default = False)
config.plugins.MyEPG.enable_wrap_multi = ConfigYesNo(default = False)
config.plugins.MyEPG.use_alternative_keys_for_timejump = ConfigYesNo(default = True)
config.plugins.MyEPG.okAction = ConfigSelection(default = "curBack", choices = [("detail",  ("Sendungsbeschreibung")), ("curCh",  ("Auf selektieren Sender umschalten")), ("curBack", ("Auf selektierten Sender umschalten / Zurück"))])
config.plugins.MyEPG.time_using_exit_settings = ConfigInteger(default = 0, limits = (0, 999))
config.plugins.MyEPG.change_time = ConfigInteger(default = 1800)
#for handling initial selections in lists
config.plugins.MyEPG.ref = ConfigText(default = "")
config.plugins.MyEPG.asktime = ConfigInteger(default = 0)
config.plugins.MyEPG.exit_time = ConfigInteger(default = 0)
config.plugins.MyEPG.ref_playing_on_exit = ConfigText(default = "")

#config.plugins.MyEPG.openType = ConfigInteger(default = 99)
#config.plugins.MyEPG.EventView = ConfigYesNo(default = False)
config.plugins.MyEPG.returnAction = ConfigInteger(default = 0) # 0 no sub started or to be started  1 sub to be started or started  2 return from sub  3 return from sub and exit
#config.plugins.MyEPG.backToTV = ConfigYesNo(default = False)


class MyEPG_config(Screen, ConfigListScreen):
	global skin_config, EPG_version
	screen_file = eEnv.resolve("${libdir}/enigma2/python/Plugins/Extensions/MyEPG/Skin/screens/screen_MyConfig.xml")
	if os.path.exists(screen_file):
		scr = open(screen_file, "r")
		skin_config = scr.read()
		scr.close()
	else:
		skin_config = ""

	def __init__(self, session):
		self.session = session
		Screen.__init__(self,session)

		global EPG_version
		EPG_version = self.modify_so("test")
		self.skin_changed = False
		self.createConfigList()

		ConfigListScreen.__init__(self, self.cflist, session = self.session, on_change = self.changedEntry)

		self["buttonred"] = Label(_("Exit"))
		self["buttongreen"] = Label(_("Save"))
		self["buttonyellow"] = Label(("Standardwerte (alle)"))
		self["buttonblue"] = Label(("Standardwert"))
		self["setupActions"] = ActionMap(["ColorActions", "SetupActions"],
			{
				"green": self.save,
				"red": self.exitWithoutSave,
				"yellow": self.resetAll,
				"blue": self.resetItem,
				"cancel": self.exitWithoutSave,
				"ok": self.save,
				"pageUp": self.keyPageUp,
				"pageDown": self.keyPageDown
			}, -4)
			
		self.onShown.append(self.setWindowTitle)
		self.skinName = "MyConfig"
		if skin_config != "":
			self.skin = skin_config
		try:
			self.config_use_old = config.plugins.MyEPG.use_MyEPG.value
		except:
			self.config_use_old = False


	def keyPageUp(self):
		act_idx = self["config"].getCurrentIndex()
		self["config"].pageUp()
		if self["config"].getCurrentIndex() > act_idx:
			self["config"].setCurrentIndex(1) # 0 is not possible, would jump to end of list

	def keyPageDown(self):
		act_idx = self["config"].getCurrentIndex()
		self["config"].pageDown()
		if self["config"].getCurrentIndex() < act_idx:
			while True:
				act_idx += 1
				try:
					self["config"].setCurrentIndex(act_idx)
					descr = self["config"].getCurrent()[1]
				except:
					act_idx -= 1
					self["config"].setCurrentIndex(act_idx)
					break

	def setWindowTitle(self):
		if EPG_version == "modified":
			epg_descr = "MyEPG (erweitertes EPG)"
		else:
			epg_descr = "Standard-EPG"
		self.setTitle(("MyEPG Setup - %s") % pluginversion + " "*10 + epg_descr)

	def createConfigList(self, add_hidden = False):
		#if reset, save or cancel also hidden items has to be in config list --> add_hidden = True
		self.sep_long = "-"*15
		sep_header = "------"
		sep_split = "-----"
		self.cflist = []
		index = 0
		if EPG_version == "modified":
			epg_descr = "MyEPG (erweitertes EPG)"
		else:
			epg_descr = "Standard - EPG"


	#page 1
		self.cflist.append(getConfigListEntry(sep_header + " " + ("MYEPG aktivieren / deaktivieren") + " " + sep_header + "  ", ConfigDescription()))

		info = ("Die ausgewählte EPG-Version wird installiert und nach Restart aktiviert.") + "\n"
		info += "Wenn die selektierte Version gleich der aktuellen ist werden nur geänderte Programmteile installiert.\n\n"
		info += "Die Installation beginnt, wenn Speichern/OK ausgeführt und danach Exit oder rote Taste.\n"
		info += "Wenn bei der Installation tatsächlich etwas verändert wurde erfolgt ein GUI-Restart."
		self.cflist.append(getConfigListEntry(("EPG-Version auswählen"),config.plugins.MyEPG.EPG_version, info))
		info = "Wenn man eigene Skinparts für MyEPG verwendet kann hier die Aktivierung der MyEPG-Standard-Skinparts verhindert werden."
		self.cflist.append(getConfigListEntry(("Standard MyEPG-Skinparts aktivieren"),config.plugins.MyEPG.activate_skins, info))
		info = ""

	#page 2
		self.cflist.append(getConfigListEntry(sep_header + " " + ("Einstellungen (nur für MyEPG gültig)") + " " + sep_header + "  ", ConfigDescription()))
		info = ("SINGLE-EPG: Nach Anfang/Ende der Liste zum Ende/Anfang springen") + "\n"
		self.cflist.append(getConfigListEntry(("SINGLE: Listenumbruch aktivieren"),config.plugins.MyEPG.enable_wrap_single, info))
		info = ("MULTI-EPG: Nach Anfang/Ende der Liste zum Ende/Anfang springen") + "\n"
		self.cflist.append(getConfigListEntry(("MULTI: Listenumbruch aktivieren"),config.plugins.MyEPG.enable_wrap_multi, info))
		info = ("MULTI-EPG: Standardmässig werden für (einstellbare) Zeitsprünge die Tasten GELB und BLAU verwendet.") + "\n"
		info += ("Hier können zusätzlich die Tasten PVR und MENU dafür aktiviert werden.")
		self.cflist.append(getConfigListEntry(("MULTI: Alternative Tasten für Zeitsprung"),config.plugins.MyEPG.use_alternative_keys_for_timejump, info))
		info = "Aktion, die beim Drücken der OK-Taste im Single- und Multi-EPG ausgeführt werden soll."
		self.cflist.append(getConfigListEntry(("Aktion bei OK"),config.plugins.MyEPG.okAction, info))
		info = ("Zuletzt selektierter Sender und Suchzeit werden beim Aufruf vom Multi- oder Single-EPG für diese Zeit (in Sekunden, 0 = unbegrenzt) nach dem letzten Beenden wiederverwendet. Danach wird die laufende Sendung selektiert.")
		self.cflist.append(getConfigListEntry(("Zeit für Anwendung letzer Sender/Zeit (in Sekunden)"),config.plugins.MyEPG.time_using_exit_settings, info))

		info = ""



	def doNothing(self, dummy = True):
		pass
	
	def changedEntry(self):
		self.createConfigList()
		curr = self["config"].getCurrent()
		self["config"].setList(self.cflist)
		if curr and curr is not None:
			self["config"].updateConfigListView(curr)
		
	def isChanged(self):
		for x in self["config"].list:
			if len(x) > 1:
				if x[1].isChanged():
					return True
		return False

	def save(self):
		curr_idx = self["config"].getCurrentIndex()
		self.createConfigList(True)
		self["config"].setList(self.cflist)
		for x in self["config"].list:
			if len(x) > 1:
				x[1].save()
				print x[0], " : ", x[1].value
		self.changedEntry()
		self["config"].setCurrentIndex(curr_idx)
		

	#def exitWithSave(self):
	#	self.save()
	#	self.close(False, self.session)
		
	def exitWithoutSave(self):
		if self.isChanged():
			self.session.openWithCallback(self.exitWithoutSaveConfirm, MessageBox, _("Really close without saving settings?"), MessageBox.TYPE_YESNO, default = True)
		else:
			self.install()
			self.close(False,self.session)

	def install(self):
		pass
		self.cnt_replaced = 0
		action = config.plugins.MyEPG.EPG_version.value
		config.plugins.MyEPG.EPG_version.setValue("stay")
		config.plugins.MyEPG.EPG_version.save()
		#config.plugins.MyEPG.EPG_version.save
		#config.plugins.MyEPG.save
		if action == "stay":
			return
		elif action == "modified":
			print "xxx23 modified "
			self.createModifiedEPG()
		elif action == "standard":
			print "xxx23 standard "
			self.createOriginalEPG()
		if self.cnt_replaced > 0:
			print "xxx23 restart"
			self.close(False,self.session)
			self.GUIrestart()


	def exitWithoutSaveConfirm(self, result):
		if result is None or result is False:
			print "[MyEPG] Cancel not confirmed."
			return
		else:
			print "[MyEPG] Cancel confirmed. Configchanges will be lost."
			self.createConfigList(True)
			self["config"].setList(self.cflist)
			for x in self["config"].list:
				if len(x) > 1:
					x[1].cancel()
			config.plugins.MyEPG.EPG_version.setValue("stay")
			self.close(False,self.session)

	def resetAll(self):
		self.session.openWithCallback(self.resetAllConfirm, MessageBox, ("MyEPG-Einstellungen auf Standardwerte zurücksetzen?"), MessageBox.TYPE_YESNO, timeout = 20, default = True)

	def resetAllConfirm(self, confirmed):
		if not confirmed:
			print "[MyEPG] Reset to defaults not confirmed."
		else:
			print "[MyEPG] Setting Configuration to defaults."
			self.createConfigList(True)
			self["config"].setList(self.cflist)
			for x in self["config"].list:
				if len(x) > 1:
					x[1].setValue(x[1].default)
			self.changedEntry()
			
	def resetItem(self):
		print "[MyEPG] Setting Configuration item to default."
		item = self["config"].getCurrent()[1]
		item.setValue(item.default)
		self.changedEntry()

	def GUIrestart(self):
		if self.cnt_modified == 0:
			import Screens.Standby
			self.session.open(Screens.Standby.TryQuitMainloop,3)

	def createFileList(self, path, start = "", end = "", contain = ""):
		filelist = []
		path = (path + "/").replace("//", "/")
		for (root, dirs, files) in os.walk(path):
			for f in files:
				#print "xxx21 ", root + "/" + f
				if f.startswith(start) and f.endswith(end) and f.find(contain) > -1:
					filelist.append((root + "/" + f, root, f))
		return filelist

	def checkFiles(self, source, target):
		#print "xxx22a ", source, target
		if not os.path.exists(source):
			return None
		if not os.path.exists(target):
			return 'new'
		sinfo = os.stat(source)[6:9]
		tinfo = os.stat(target)[6:9]
		#print "xxx22b ", sinfo, tinfo
		if sinfo[2] > tinfo[2]:
			return 'newer'
		elif sinfo[2] < tinfo[2]:
			return 'older'
		elif sinfo[0] == tinfo[0]:
			return 'equal'
		else:
			return 'changed'
		

	def createModifiedEPG(self):
		self.cnt_replaced = 0

		#programs
		sourceDir = '/usr/lib/enigma2/python/Plugins/Extensions/MyEPG/templates/mod/'
		sourceFiles = self.createFileList(sourceDir)
		targetDir = '/usr/lib/enigma2/'
		for pathfile, path, file in sourceFiles:
			targetpath = (targetDir + path.replace(sourceDir, "") + "/").replace("//", "/").replace("//", "/")
			targetfile = targetpath + file
			check = self.checkFiles(pathfile, targetfile)
			if check in ('new', 'newer'):
				if not os.path.exists(targetpath):
					os.makedirs(targetpath)
				copy(pathfile, targetfile)
				self.cnt_replaced += 1
				timestamp = os.stat(pathfile).st_mtime
				os.utime(targetfile, (timestamp, timestamp))
				print "xxx22c ", pathfile, targetfile

		#skins
		used_skin = config.skin.primary_skin.value
		used_skin = "/share/enigma2/" + used_skin[0:used_skin.find("/")] + "/"
		#config.skin.primary_skin=Nemesis_FHD/skin.xml
		#/share/enigma2/Nemesis_FHD/
		sourceDir = '/usr/lib/enigma2/python/Plugins/Extensions/MyEPG/Skin/skins/'

		sourceFiles = self.createFileList(sourceDir, "skin_wefr_MyEPG", ".xml")
		targetDir = used_skin + "allScreens/"
		for pathfile, path, file in sourceFiles:
			targetpath = (targetDir + path.replace(sourceDir, "") + "/").replace("//", "/").replace("//", "/")
			targetfile = targetpath + file
			check = self.checkFiles(pathfile, targetfile)
			if check in ('new', 'newer'):
				if not os.path.exists(targetpath):
					os.makedirs(targetpath)
				copy(pathfile, targetfile)
				self.cnt_replaced += 1
				timestamp = os.stat(pathfile).st_mtime
				os.utime(targetfile, (timestamp, timestamp))
				print "xxx22d ", pathfile, targetfile
				
		if config.plugins.MyEPG.activate_skins.value:
			targetDir = used_skin + "mySkin/"
			for pathfile, path, file in sourceFiles:
				targetpath = (targetDir + path.replace(sourceDir, "") + "/").replace("//", "/").replace("//", "/")
				targetfile = targetpath + file
				check = self.checkFiles(pathfile, targetfile)
				if check in ('new', 'newer'):
					if not os.path.exists(targetpath):
						os.makedirs(targetpath)
					copy(pathfile, targetfile)
					self.cnt_replaced += 1
					timestamp = os.stat(pathfile).st_mtime
					os.utime(targetfile, (timestamp, timestamp))
					print "xxx22d ", pathfile, targetfile

		# so-Dateien
		cnt = self.modify_so(True)
		self.cnt_replaced += cnt

	def createOriginalEPG(self):
		# chancel modifications in InfoBar.so and InfoBarGenerics.so
		self.cnt_replaced = 0
		self.cnt_replaced = self.modify_so(False)

	def modify_so(self, modify = True):
		self.cnt_modified = 0
		moddatafile = "/usr/lib/enigma2/python/Plugins/Extensions/MyEPG/templates/Modifications_so.data"
		if not os.path.exists(moddatafile):
			return
		mods = []
		with open(moddatafile, "r") as f:
			for z in f:
				tmp = (z + ";;;").split(";")
				datnam = tmp[0].strip()
				old = tmp[1].strip()
				new = tmp[2].strip()
				new += ("x" * max(1, len(new) - len(old)))
				new = new[0:len(old)]  # len of new and old must be equal
				if not modify == True: # back to standard
					old, new = new, old
				mods.append([datnam, old, new])
		mods.sort()
		mods.append(["", "", ""])
		print "xxx46 ", mods

		cnt_change1 = 0
		cnt_change2 = 0
		file_old = ""
		
		for file, old, new in mods:
			t = time.time()
			if file <> file_old:
				if cnt_change1 > 0 and modify != "test":
					with open(file_old, "wb") as f:
						f.write(content)
						f.close()
						print "xxx48 write file ", file_old
				cnt_change2 += cnt_change1
				cnt_change1 = 0

			if file == "" or not os.path.exists(file):
				continue
			if file <> file_old:
				with open(file, "rb") as f:
					content = f.read()
				f.close()
			n = content.find(chr(0) + old + chr(0))
			print n, old
			if n > -1:
				cnt_change1 += 1
				if modify != "test":
					content = content.replace(chr(0)+old+chr(0), chr(0)+new+chr(0))
			print "xxx49 file ", file, cnt_change1
			file_old = file

		print "xxx47 Zeit: ", time.time() - t
		if modify == "test":
			if cnt_change2 > 0:
				return "modified"
			else:
				return "standard"
		else:
			self.cnt_modified = cnt_change2
			return cnt_change2



class startEPG: # not used!
	def __init__(self):
		self.keys = ["370/1", "370/0", "103/1x", "108/1x"]
		self.previousflag = 0
		self.prev_key = "/"
		self.blocked = False
		#eActionMap.getInstance().bindAction('', -0x7FFFFFFF, self.getKey)

		#flags: 0 press m   1 release b   2 repeated r   3 long l
	def getKey(self, inkey, flag):
		#print "WeFraJo MyEPG startEPG xxx17 ", "x"+str(key)+"x", "x"+str(flag)+"x"
		key = str(inkey) + "/" + str(flag)
		if flag == 2:
			return 0
		#print "WeFraJo MyEPG startEPG xxx17 ", key
		if not key in self.keys:
			return 0
		if flag == 3:
			self.blocked = True
		else:
			self.blocked = False
		if self.blocked and flag == 1:
			return 0
		#self.key = key
		self.startOnKeys(key)
		return 1

	def startOnKeys(self, key):
		if key == "370/0":   #subtitles
			self.startTest()
		if key == "370/1":   #subtitles
			pass
		#elif key == "103/1":   #up
		#	self.startMulti()
		#elif key == "108/1":   #down
		#		self.startSingle()


def autostart(reason, **kwargs):
	if kwargs.has_key("session") and reason == 0:
		return
		startEPG()

def startSetup(session, **kwargs):
	print "[MyEPG] start configuration"
	session.open(MyEPG_config)

def Plugins(**kwargs):
	a = []
	a.append(PluginDescriptor(where = PluginDescriptor.WHERE_SESSIONSTART, fnc = autostart))
	a.append(PluginDescriptor(name = ("MyEPG (Setup)"), description = ("Enhancement for Standard EPG"), where = [PluginDescriptor.WHERE_PLUGINMENU, PluginDescriptor.WHERE_EXTENSIONSMENU], fnc=startSetup))
	return a


