# coding=utf-8
from __init__ import MATCH_ANY, MATCH_BEGIN, MATCH_EXACT

from enigma import eEPGCache, eServiceReference, eServiceCenter, RT_HALIGN_LEFT, \
		RT_HALIGN_RIGHT, eListboxPythonMultiContent, eTimer

from Tools.LoadPixmap import LoadPixmap
from Tools.Directories import fileExists, resolveFilename, SCOPE_PLUGINS
from ServiceReference import ServiceReference

from EPGSearchSetup import EPGSearchSetup
from Screens.ChannelSelection import SimpleChannelSelection
from Screens.ChoiceBox import ChoiceBox
from Screens.MyEpgSelecti import MyEPGSelecti
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from Screens.HelpMenu import HelpableScreen
from Screens.VirtualKeyBoard import VirtualKeyBoard
from Screens.InfoBar import InfoBar 
from Components.Sources.StaticText import StaticText
from Components.ActionMap import ActionMap, HelpableActionMap
from Components.Button import Button
from Components.config import config
from Components.EpgList import EPGList, EPG_TYPE_SINGLE, EPG_TYPE_MULTI
from Components.TimerList import TimerList
from Components.Sources.ServiceEvent import ServiceEvent
from Components.Sources.Event import Event
from Components.Sources.ExtEvent import ExtEvent
from Components.Sources.Event import Event
from Components.Sources.ServiceEvent import ServiceEvent

from time import localtime
from operator import itemgetter, eq, ne
import re
import os
from Components.Label import Label
import skin

# AutoTimer installed?
try:
	from Plugins.Extensions.AutoTimer.AutoTimerEditor import \
			addAutotimerFromEvent, addAutotimerFromSearchString
	autoTimerAvailable = True
except ImportError:
	autoTimerAvailable = False

# Overwrite EPGSelection.__init__ with our modified one
baseEPGSelection__init__ = None
def EPGSelectionInit():
	global baseEPGSelection__init__
	if baseEPGSelection__init__ is None:
		baseEPGSelection__init__ = MyEPGSelecti.__init__
	MyEPGSelecti.__init__ = EPGSelection__init__

# Modified EPGSelection __init__
def EPGSelection__init__(self, session, service, zapFunc=None, eventid=None, bouquetChangeCB=None, serviceChangeCB=None, isEPGBar=None, switchBouquet=None, EPGNumberZap=None, togglePiP=None):
	baseEPGSelection__init__(self, session, service, zapFunc, eventid, bouquetChangeCB, serviceChangeCB, isEPGBar, switchBouquet, EPGNumberZap, togglePiP)

	if self.type != EPG_TYPE_MULTI and config.plugins.MyEpgSearch.add_search_to_epg.value:
		def bluePressed():
			cur = self["list"].getCurrent()
			if cur[0] is not None:
				name = cur[0].getEventName()
			else:
				name = ''
			self.session.open(EPGSearch, name, False)
		self["SelectedEvent"] = StaticText()
		self["ExtEvent"] = ExtEvent()
		self["epgsearch_actions"] = ActionMap(["EPGSelectActions"],
				{
					"blue": bluePressed,
				})
		self["key_blue"].text = _("Search")
		global zapFunc2
		zapFunc2 = zapFunc #+ WeFraJo
		#print "xxx50 zapFunc2"


class EPGSearchList(EPGList):
	def __init__(self, type=EPG_TYPE_SINGLE, selChangedCB=None, timer=None):
		EPGList.__init__(self, type, selChangedCB, timer)
		self.show_short_description = config.plugins.MyEpgSearch.show_short_description.value
		self.l.setBuildFunc(self.buildEPGSearchEntry)
		if self.show_short_description:
			self.l.setItemHeight(int(skin.parameters.get("EPGSearchItemHeightBig", (50,))[0]))
		else: 
			self.l.setItemHeight(int(skin.parameters.get("EPGSearchItemHeightDefault", (30,))[0]))

	def buildEPGSearchEntry(self, service, eventId, beginTime, duration, EventName):
		(clock_pic_local, rec1) = self.getPixmapForEntry(service, eventId, beginTime, duration)
		rec2 = False
		r1 = self.weekday_rect
		r2 = self.datetime_rect
		r3 = self.descr_rect
		t = localtime(beginTime)
		serviceref = ServiceReference(service) # for Servicename
		x1,y1,w1,h1 = skin.parameters.get("EPGlistText1", (r1.left(), r1.top(), r1.width(), r1.height()))
		x2,y2,w2,h2 = skin.parameters.get("EPGlistText2", (r2.left(), r2.top(), r2.width(), r2.height()))
		res = [
			None, # no private data needed
			(eListboxPythonMultiContent.TYPE_TEXT, x1, y1, w1, h1, 0, RT_HALIGN_RIGHT, self.days[t[6]]),
			(eListboxPythonMultiContent.TYPE_TEXT, x2, y2, w2, h2, 0, RT_HALIGN_RIGHT, "%02d.%02d.  %02d:%02d"%(t[2],t[1],t[3],t[4]))		
			]
		if rec1 or rec2:
			if rec1:
				clock_pic = clock_pic_local
				if rec2:
					clock_pic_partnerbox = getRemoteClockPixmap(self,service, beginTime, duration, eventId)
			else:
				clock_pic = getRemoteClockPixmap(self,service, beginTime, duration, eventId)
			if rec1 and rec2:
				x3,y3,w3,h3 = skin.parameters.get("EPGlistRecImage", (r3.left(), r3.top(), 21, 21))
				x4,y4,w4,h4 = skin.parameters.get("EPGlistRecText", (r3.left() + 25, r3.top(), r3.width(), r3.height()))
				res.extend((
						(eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, x3, y3, w3, h3, clock_pic),
						(eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, x3 + 25, y3, w3, h3, clock_pic_partnerbox),
						(eListboxPythonMultiContent.TYPE_TEXT, x4, y4, w4, h4, 0, RT_HALIGN_LEFT, serviceref.getServiceName() + ": " + EventName)))
			else:
				x3,y3,w3,h3 = skin.parameters.get("EPGlistRecImage", (r3.left(), r3.top(), 21, 21))
				x4,y4,w4,h4 = skin.parameters.get("EPGlistRecText", (r3.left() + 25, r3.top(), r3.width(), r3.height()))			
				res.extend((
						(eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, x3, y3, w3, h3, clock_pic),
						(eListboxPythonMultiContent.TYPE_TEXT, x4, y4, w4, h4, 0, RT_HALIGN_LEFT, serviceref.getServiceName() + ": " + EventName)))
		else:
			x3,y3,w3,h3 = skin.parameters.get("EPGlistNonRecText", (r3.left(), r3.top(), r3.width(), r3.height()))
			res.append((eListboxPythonMultiContent.TYPE_TEXT, x3, y3, w3, h3, 0, RT_HALIGN_LEFT, serviceref.getServiceName() + ": " + EventName))
		if self.show_short_description:
			event = self.epgcache.lookupEventId(serviceref.ref, eventId)
			if event:
				horizontal_offset = self.l.getItemSize().height()/2
				x5,y5,w5,h5 = skin.parameters.get("EPGlistText3", (r3.left(), r3.top(), r3.width(), r3.height()))
				res.append((eListboxPythonMultiContent.TYPE_TEXT, x5, y5 + horizontal_offset, w5, h5, 0, RT_HALIGN_LEFT, event.getShortDescription()))
		return res

# main class of plugin
class EPGSearch(MyEPGSelecti, HelpableScreen):
	def __init__(self, session, *args):
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		self.skinName = ["MyEPGSearch", "MyEPGSelection"]
		self["popup"] = Label()
		self["popup"].hide()
		self.hidePopup = eTimer()
		
		self["SelectedEvent"] = StaticText()
		self["ExtEvent"] = ExtEvent()
		
		self.searchargs = args
		self.currSearch = ""
		self.searchType = eEPGCache.PARTIAL_TITLE_SEARCH #default search type
		self.search_string = ""

		self.bouquetservices = []
		self.limit_to_bouquet = config.plugins.MyEpgSearch.limit_to_bouquet.value
		self.show_short_description = config.plugins.MyEpgSearch.show_short_description.value
		self.match_type = config.plugins.MyEpgSearch.match_type.value

		# XXX: we lose sort begin/end here
		self["key_yellow"] = Button(_("New Search"))
		self["key_blue"] = Button(_("History"))

# begin stripped copy of EPGSelection.__init__
		self.isNotSearch = False
		self.switchBouquet = None
		self.bouquetChangeCB = None
		self.serviceChangeCB = None
		self.ask_time = -1 #now
		self["key_red"] = Button(_("Edit Search"))
		self.closeRecursive = False
		self.saved_title = None
		self["Service"] = ServiceEvent()
		self["Event"] = Event()
		self.type = EPG_TYPE_SINGLE
		self.currentService=None
	#+ start WeFraJo
			# variables additional defined in modified version ???
		try:
			self.ref_currentPlaying = self.session.nav.getCurrentlyPlayingServiceReference().toString()
		except:
			self.ref_currentPlaying = ""
	#+ end WeFraJo
		#self.ref_currentPlaying = self.session.nav.getCurrentlyPlayingServiceReference().toString() #- WeFraJo
		self.zapFunc = None
		self.sort_type = 0
		self["key_green"] = Button(_("Add timer"))
		self.key_green_choice = self.ADD_TIMER
		self.key_red_choice = self.EMPTY
		self["list"] = EPGSearchList(type = self.type, selChangedCB = self.onSelectionChanged, timer = session.nav.RecordTimer)
		self["actions"] = HelpableActionMap(self, "EPGSelectActions",
			{
				"timerAdd": self.timerAdd,
				"timerEnableDisable": self.timerEnableDisable,
				"instantToggleTimerState" : self.instantToggleTimerState,
				"instantTimer": self.addInstantTimer,
				"yellow": self.yellowButtonPressed,
				"blue": self.blueButtonPressed,
				"infoxxx": self.infoKeyPressed,
				"red": self.redButtonPressed,
				"nextBouquet": self.nextBouquet, # just used in multi epg yet
				"prevBouquet": self.prevBouquet, # just used in multi epg yet
				"nextService": self.nextService, # just used in single epg yet
				"prevService": self.prevService, # just used in single epg yet
			}, -1)
		self["MenuActions"] = HelpableActionMap(self, "MenuActions",
			{
				"menu": (self.menu, _("Settings")),
			}, -1)
		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
			{
				"cancel": (self.closeScreen, _("Exit")),
				"ok": (self.eventSelected, _("Select")),
			}, -1)
		self["NumberActions"] = HelpableActionMap(self, "NumberActions",
			{
				"0": (self.Number0, _("Reset search result list")),
				"1": (self.Number1, _("Remove Channel from result list")),
				"3": (self.Number3, _("Keep only this Channel in result list")),
				"4": (self.Number4, _("Remove Event from result list")),
				"5": (self.Number5, _("Toggle: show event short description in result list")),
				"6": (self.Number6, _("Keep only this Event in result list")),
				"7": (self.Number7, _("Match of search string: exact, from begin or any substring")),
				"9": (self.Number9, _("Toggle: search all Channels or only Channels in Bouquet")),
			}, -1)

		self.initTimer = eTimer()
		self.initTimer.callback.append(self.createFinished)

		self["actions"].csel = self
		self.onLayoutFinish.append(self.onCreate)
# end stripped copy of EPGSelection.__init__

	def onCreate(self):
		self.setTitle(_("EPG Search"))

		if self.searchargs:
			#self.searchEPG(*self.searchargs)
			self.initTimer.start(100, True)
		else:
			l = self["list"]
			l.recalcEntrySize()
			l.list = []
			l.l.setList(l.list)
			del self.searchargs

	def createFinished(self):
		self.initTimer.stop()
		self.searchEPGWithConfig(*self.searchargs)
		del self.searchargs

	def createPopUpInfo(self):
		# Show popup to inform user about pre-selected options.
		message = _("Search all channels")
		if self.limit_to_bouquet:
			message = _("Limit to bouquet")
		if self.match_type == MATCH_ANY:
			message = message + "\n" + _("Match any substring")
		elif self.match_type == MATCH_BEGIN:
			message = message + "\n" + _("Match beginning of a word")
		elif self.match_type == MATCH_EXACT:
			message = message + "\n" + _("Match exactly")
		self.ShowPopUp(message)

	def closeScreen(self):
		# Save our history
		self.hidePopup.stop()
		config.plugins.MyEpgSearch.save()
		#self.last_reference = None #+ WeFraJo
		MyEPGSelecti.closeScreen(self)

	def yellowButtonPressed(self):
		self.session.openWithCallback(
			self.searchEPGWrapper,
			VirtualKeyBoard,
			title = _("Enter text to search for")
		)
		
	def redButtonPressed(self):
		self.session.openWithCallback(
			self.searchEPGWrapper,
			VirtualKeyBoard,
			title = _("Enter text to search for"),
			text = self.currSearch
		)

	def menu(self):
		options = [
			(_("Import from Timer"), self.importFromTimer),
			(_("Import from EPG"), self.importFromEPG),
		]

		if autoTimerAvailable:
			options.extend((
				(_("Import from AutoTimer"), self.importFromAutoTimer),
				(_("Save search as AutoTimer"), self.addAutoTimer),
				(_("Export selected as AutoTimer"), self.exportAutoTimer),
			))
		if fileExists(resolveFilename(SCOPE_PLUGINS, "Extensions/IMDb/plugin.py")):
			options.append((_("Open selected in IMDb"), self.openImdb))
		options.append(
				(_("Setup"), self.setup)
		)

		self.session.openWithCallback(
			self.menuCallback,
			ChoiceBox,
			list = options
		)

	def menuCallback(self, ret):
		ret and ret[1]()

	def getBouquetServices(self):
		services = []
		InfoBar_Instance = InfoBar.instance
		servicelist = InfoBar_Instance.servicelist
		bouquets = servicelist.getBouquetList()
	#+ start WeFraJo
		if config.plugins.MyEpgSearch.limit_to_active_bouquet.value:
			current_bq = self.getCurrentBouquetName()
		else:
			current_bq = None
	#+ end WeFraJo
		if bouquets and bouquets is not None:
			for bouquet in bouquets:
				if current_bq is not None and bouquet[0] != current_bq: #+ WeFraJo
					continue   #+ WeFraJo
				servicelist = eServiceCenter.getInstance().list(bouquet[1])
				if not servicelist is None:
					while True:
						service = servicelist.getNext()
						if not service.valid(): #check if end of list
							break
						if service.flags & (eServiceReference.isDirectory | eServiceReference.isMarker): #ignore non playable services
							continue
						services.append(service.toString())
		return services

#+ start WeFraJo
	def getCurrentBouquetName(self):  #+ WeFrajo
		lastroot = config.tv.lastroot.value.strip()
		bouquet = lastroot.strip()[:-1].split(";")[-1]
		bq_file = "/var/etc/enigma2/" + bouquet.split('"')[1]
		name = None
		if os.path.exists(bq_file):
			f = open(bq_file, "r")
			while True:
				a = f.readline()
				if not a:
					break
				a = a.strip()
				if a.startswith("#NAME"):
					name = a.replace("#NAME", "").strip()
					break
			f.close()
		return name

	def eventSelected(self):
		if config.plugins.MyEpgSearch.okAction.value == "detail":
			self.infoKeyPressed()
		else:
			self.zapTo()


	def zapTo(self):
		if self.type == EPG_TYPE_SINGLE:
			lst = self['list']
			count = lst.getCurrentChangeCount()
			if 1==1:
				self.closeRecursive = True
				ref = lst.getCurrent()[1]
				ref2 = str(ref)
				ref = ServiceReference(ref2)

				if not ref:
					ref = self.session.nav.getCurrentlyPlayingServiceReference()
				check_bouquet = True
				#print "xxxs1 ", ref
				#print "xxxs1 ", ref.ref
				zapFunc2(ref.ref, check_bouquet) #change to selected service
				if not config.usage.servicelist_preview_mode.value:
					self.zapActions(ref2)

	def zapActions(self, ref):
		if ref and (ref == self.ref_currentPlaying): 
			if config.plugins.MyEpgSearch.okAction.value == "curBack":
				self.closeScreen()
		else: 
			self.ref_currentPlaying = ref
			#self['key_red'].setText('Zurück zu TV')
			#self['zapTo'].setText('Zurück zu TV')



#+ end WeFraJo

	def ShowPopUp(self, message):
		if int(config.plugins.MyEpgSearch.show_popup_time .value) > 0:
			self["popup"].setText(_(message))
			self["popup"].show()
			show_time = int(config.plugins.MyEpgSearch.show_popup_time .value)
			self.hidePopup.callback.append(self.HidePopUp)
			self.hidePopup.start(show_time * 1000)

	def HidePopUp(self):
		self["popup"].hide()

	def Number0(self):
		self.CheckNumber(0)
	def Number1(self):
		self.CheckNumber(1)
	def Number3(self):
		self.CheckNumber(3)
	def Number4(self):
		self.CheckNumber(4)
	def Number5(self):
		self.CheckNumber(5)
	def Number6(self):
		self.CheckNumber(6)
	def Number7(self):
		self.CheckNumber(7)
	def Number9(self):
		self.CheckNumber(9)

	def CheckNumber(self, number):
		filtertype = None
		operator = None
		
		if number == 0:
			self.searchEPG(searchString = self.currSearch, searchSave = False)
			self.ShowPopUp("Removed all filters")
			return
		elif number == 1:
			filtertype = "channel"
			operator = ne
		elif number == 3:
			filtertype = "channel"
			operator = eq
		elif number == 4:
			filtertype = "event"
			operator = ne
		elif number == 5:
			self.show_short_description = not self.show_short_description
			l = self["list"]
			l.show_short_description = self.show_short_description
			if self.show_short_description:
				l.l.setItemHeight(int(skin.parameters.get("EPGSearchItemHeightBig", (50,))[0]))
			else: 
				l.l.setItemHeight(int(skin.parameters.get("EPGSearchItemHeightDefault", (30,))[0]))
			l.recalcEntrySize()
			ret = l.list
			l.list = ret
			l.l.setList(ret)
			message = _("Hide event short description")
			if self.show_short_description:
				message = _("Show event short description")
			self.ShowPopUp(message)
			return
		elif number == 6:
			filtertype = "event"
			operator = eq
		elif number == 7:
			if self.match_type == MATCH_ANY:
				self.match_type = MATCH_BEGIN
				message = _("Match beginning of a word")
			elif self.match_type == MATCH_BEGIN:
				self.match_type = MATCH_EXACT
				message = _("Match exactly")
			elif self.match_type == MATCH_EXACT:
				self.match_type = MATCH_ANY
				message = _("Match any substring")
			self.searchEPG(searchString = self.currSearch, searchSave = False)
			self.ShowPopUp(message)
			return
		elif number == 9:
			self.limit_to_bouquet = not self.limit_to_bouquet
			self.searchEPG(searchString = self.currSearch, searchSave = False)
			message = _("Search all channels")
			if self.limit_to_bouquet:
				message = _("Limit to bouquet")
			self.ShowPopUp(message)
			return
		if filtertype and operator:
			self.filterResultList(filtertype, operator)
			message = _("Filter: remove") + " " + _(filtertype)
			if operator == eq:
				message = _("Filter: keep") + " " + _(filtertype)
			self.ShowPopUp(message)

	def filterResultList(self, filtertype, operator):
		cur = self['list'].getCurrent()
		if cur is None:
			return
		if cur[0] is not None and cur[1] is not None:
			serviceEvent = cur[0]
			eServiceRef = cur[1]
		else:
			return
		l = self["list"]
		ret = l.list
		if filtertype == "channel":
			X = 0
			filterString = str(eServiceRef)
		if filtertype == "event":
			X = 4
			filterString = serviceEvent.getEventName()
		for entry in reversed(ret):
			if not operator(entry[X].lower(), filterString.lower()):
				ret.remove(entry)
		l.recalcEntrySize()
		l.list = ret
		l.l.setList(ret)

	def applyFilterOptions(self, ret, searchString):
		if not self.limit_to_bouquet and self.match_type == MATCH_ANY:
			return ret
		for entry in reversed(ret):
			channel_ref = entry[0]
			if self.limit_to_bouquet and not self.bouquetservices:
				self.bouquetservices = self.getBouquetServices()
			if self.limit_to_bouquet and not channel_ref in self.bouquetservices:
				ret.remove(entry)
				continue
			event_name = entry[4]
			if not self.IsSearchStringMatching(searchString, event_name, self.match_type):
				ret.remove(entry)
				continue
		return ret

	def IsSearchStringMatching(self, searchString, event_name, match_type):
		if match_type == MATCH_ANY:
			return True
		my_regex = r'\b' + re.escape(searchString)
		if match_type == MATCH_BEGIN and re.search(my_regex, event_name, re.IGNORECASE | re.UNICODE):
			return True
		elif match_type == MATCH_EXACT and re.search(my_regex + r'\b', event_name, re.IGNORECASE | re.UNICODE):
			return True
		else:
			return False

	def importFromTimer(self):
		self.session.openWithCallback(
			self.searchEPGWrapper,
			EPGSearchTimerImport
		)

	def importFromEPG(self):
		self.session.openWithCallback(
			self.searchEPGWrapper,
			EPGSearchChannelSelection
		)

	def importFromAutoTimer(self):
		removeInstance = False
		try:
			# Import Instance
			from Plugins.Extensions.AutoTimer.plugin import autotimer

			if autotimer is None:
				removeInstance = True
				# Create an instance
				from Plugins.Extensions.AutoTimer.AutoTimer import AutoTimer
				autotimer = AutoTimer()

			# Read in configuration
			autotimer.readXml()
		except Exception as e:
			self.session.open(
				MessageBox,
				_("Could not read AutoTimer timer list: %s") % e,
				type = MessageBox.TYPE_ERROR
			)
		else:
			# Fetch match strings
			# XXX: we could use the timer title as description
			options = [(x.match, x.match) for x in autotimer.getTimerList()]

			self.session.openWithCallback(
				self.searchEPGWrapper,
				ChoiceBox,
				title = _("Select text to search for"),
				list = options
			)
		finally:
			# Remove instance if there wasn't one before
			if removeInstance:
				autotimer = None

	def addAutoTimer(self):
		addAutotimerFromSearchString(self.session, self.currSearch)

	def exportAutoTimer(self):
		cur = self['list'].getCurrent()
		if cur is None:
			return
		addAutotimerFromEvent(self.session, cur[0], cur[1])

	def openImdb(self):
		cur = self['list'].getCurrent()
		if cur is None:
			return
		try:
			from Plugins.Extensions.IMDb.plugin import IMDB
			self.session.open(IMDB, cur[0].getEventName())
		except ImportError as ie:
			pass

	def setup(self):
		self.session.open(EPGSearchSetup)

	def blueButtonPressed(self):
		options = [(x, x) for x in config.plugins.MyEpgSearch.history.value]

		if options:
			self.session.openWithCallback(
				self.searchEPGWrapper,
				ChoiceBox,
				title = _("Select text to search for"),
				list = options
			)
		else:
			self.session.open(
				MessageBox,
				_("No history"),
				type = MessageBox.TYPE_INFO
			)

	def searchEPGWrapper(self, ret = None, searchSave = True):
		if ret:
			if isinstance(ret, str):
				search_str = ret
			if isinstance(ret, (tuple, list)):
				search_str = ret[1]
			if config.plugins.MyEpgSearch.search_type.value == "ask":
				self.search_string = search_str
				options = [
					(_("Title"), "title"),
					(_("Short description"), "shortdesc"),
					(_("Title and short description"), "title_shortdesc"),
					(_("Extended description"), "extdesc"),
					(_("Title, short and extended description"), "full")]
				self.tmp_save_search = searchSave
				self.session.openWithCallback(
					self.searchTypeSelected,
					ChoiceBox,
					title = _("Select EPG search typ"),
					list = options
				)
			else:
				self.setSearchType()
				self.searchEPG(search_str, searchSave = searchSave)

	def setSearchType(self):
		if config.plugins.MyEpgSearch.search_type.value == "extdesc":
			self.searchType = eEPGCache.EXTENDED_DESCRIPTION_SEARCH
		elif config.plugins.MyEpgSearch.search_type.value == "full":
			self.searchType = eEPGCache.FULL_DESCRIPTION_SEARCH
		elif config.plugins.MyEpgSearch.search_type.value == "shortdesc":
			self.searchType = eEPGCache.SHORT_DESCRIPTION_SEARCH
		elif config.plugins.MyEpgSearch.search_type.value == "title_shortdesc":
			self.searchType = eEPGCache.TITLE_SHORT_DESCRIPTION_SEARCH
		else:
			self.searchType = eEPGCache.PARTIAL_TITLE_SEARCH

	def searchTypeSelected(self, ret):
		self.searchType = eEPGCache.PARTIAL_TITLE_SEARCH
		if ret:
			if ret[1] == "extdesc":
				self.searchType = eEPGCache.EXTENDED_DESCRIPTION_SEARCH
			elif ret[1] == "full":
				self.searchType = eEPGCache.FULL_DESCRIPTION_SEARCH
			elif ret[1] == "shortdesc":
				self.searchType = eEPGCache.SHORT_DESCRIPTION_SEARCH
			elif ret[1] == "title_shortdesc":
				self.searchType = eEPGCache.TITLE_SHORT_DESCRIPTION_SEARCH
		self.searchEPG(self.search_string, searchSave = self.tmp_save_search)

	def searchEPGWithConfig(self, searchString = None, searchSave = True):
		if searchString:
			self.searchEPGWrapper(searchString, searchSave)

	def searchEPG(self, searchString = None, searchSave = True):
		if searchString:
			self.currSearch = searchString
			if searchSave or config.plugins.MyEpgSearch.save_any_search_string.value:
				# Maintain history
				history = config.plugins.MyEpgSearch.history.value
				if searchString not in history:
					history.insert(0, searchString)
					maxLen = config.plugins.MyEpgSearch.history_length.value
					if len(history) > maxLen:
						del history[maxLen:]
				else:
					history.remove(searchString)
					history.insert(0, searchString)

			case_type = eEPGCache.NO_CASE_CHECK
			if config.plugins.MyEpgSearch.search_casetype.value:
				case_type = eEPGCache.CASE_CHECK
			# Search EPG, default to empty list
			epgcache = eEPGCache.getInstance() # XXX: the EPGList also keeps an instance of the cache but we better make sure that we get what we want :-)
			ret = epgcache.search(('RIBDT', 1000, self.searchType, searchString , case_type)) or []
			ret = self.applyFilterOptions(ret, searchString)
			ret.sort(key=itemgetter(2)) # sort by time

			# Update List
			l = self["list"]
			l.recalcEntrySize()
			l.list = ret
			l.l.setList(ret)
			self.createPopUpInfo()

class EPGSearchTimerImport(Screen):
	def __init__(self, session):
		Screen.__init__(self, session)
		self.skinName = ["EPGSearchTimerImport", "TimerEditList"]

		self.list = []
		self.fillTimerList()

		self.show_only_running = False
		self["Event"] = Event()
		self["ServiceEvent"] = ServiceEvent()

		self["timerlist"] = TimerList(self.list)

		self["key_red"] = Button(_("Cancel"))
		self["key_green"] = Button(_("OK"))
		self["key_yellow"] = Button("")
		self["key_blue"] = Button("")

		self["actions"] = ActionMap(["OkCancelActions", "ColorActions", "DirectionActions"],
		{
			"ok": self.search,
			"cancel": self.cancel,
			"green": self.search,
			"red": self.cancel,
			"left": self.left,
			"right": self.right,
			"up": self.up,
			"down": self.down,
		}, -1)
		self.onLayoutFinish.append(self.setCustomTitle)

	def setCustomTitle(self):
		self.setTitle(_("Select a timer to search"))
		if config.usage.timerlist_show_epg.value:
			self.updateState()

	def fillTimerList(self):
		l = self.list
		del l[:]

		for timer in self.session.nav.RecordTimer.timer_list:
			l.append((timer, False))

		for timer in self.session.nav.RecordTimer.processed_timers:
			l.append((timer, True))
		l.sort(key = lambda x: x[0].begin)

	def search(self):
		cur = self["timerlist"].getCurrent()
		if cur:
			self.close(cur.name)

	def cancel(self):
		self.close(None)

	def up(self):
		self["timerlist"].instance.moveSelection(self["timerlist"].instance.moveUp)
		self.updateState()
		
	def down(self):
		self["timerlist"].instance.moveSelection(self["timerlist"].instance.moveDown)
		self.updateState()

	def left(self):
		self["timerlist"].instance.moveSelection(self["timerlist"].instance.pageUp)
		self.updateState()
		
	def right(self):
		self["timerlist"].instance.moveSelection(self["timerlist"].instance.pageDown)
		self.updateState()

	def updateState(self):
		cur = self["timerlist"].getCurrent()
		if cur and config.usage.timerlist_show_epg.value:
			event = self.getEPGEvent(cur)
			if event:
				self["Event"].newEvent(event)
				self["ServiceEvent"].newService(cur.service_ref.ref)

	def getEPGEvent(self, timer):
		event = None
		if timer:
			event_id = None
			epgcache = eEPGCache.getInstance()
			if hasattr(timer, "eit") and timer.eit:
				event = epgcache.lookupEventId(timer.service_ref.ref, timer.eit)
			if not event:
				if isinstance(timer.service_ref, str):
					ref = timer.service_ref
				else:
					ref = timer.service_ref.ref.toString()
				begin = timer.begin + config.recording.margin_before.value*60
				duration = (timer.end - begin - config.recording.margin_after.value*60) / 60
				if duration <= 0:
					duration = 30 # it seems to be a reminder or a justplay timer without end time, so search epg events for the next 30 min
				list = epgcache.lookupEvent([ 'IBDT', (ref, 0, begin, duration) ])
				if len(list):
					for epgevent in list:
						if timer.name.startswith(epgevent[3]):
							event_id = epgevent[0]
							break
					if not event_id and timer.begin != timer.end: # no match at title search --> search in time span
						for epgevent in list:
							if  timer.end >= (begin + epgevent[2]) and timer.begin <= epgevent[1]:
								event_id = epgevent[0]
								break
					if event_id:
						event = epgcache.lookupEventId(timer.service_ref.ref, event_id)
		return event

class EPGSearchChannelSelection(SimpleChannelSelection):
	def __init__(self, session):
		SimpleChannelSelection.__init__(self, session, _("Channel Selection"))
		self.skinName = ["EPGSearchChannelSelection", "SimpleChannelSelection"]

		self["ChannelSelectEPGActions"] = ActionMap(["ChannelSelectEPGActions"],
		{
				"showEPGList": self.channelSelected
		})

	def channelSelected(self):
		ref = self.getCurrentSelection()
		if (ref.flags & 7) == 7:
			self.enterPath(ref)
		elif not (ref.flags & eServiceReference.isMarker):
			self.session.openWithCallback(
				self.epgClosed,
				EPGSearchEPGSelection,
				ref,
				False
			)

	def epgClosed(self, ret = None):
		if ret:
			self.close(ret)

class EPGSearchEPGSelection(MyEPGSelecti):
	def __init__(self, session, ref, openPlugin):
		MyEPGSelecti.__init__(self, session, ref)
		self.skinName = ["EPGSearchEPGSelection", "EPGSelection"]
		self["key_green"].text = _("Search")
		self.openPlugin = openPlugin

	def infoKeyPressed(self):
		self.timerAdd()

	def timerAdd(self):
		cur = self["list"].getCurrent()
		evt = cur[0]
		sref = cur[1]
		if not evt:
			return

		if self.openPlugin:
			self.session.open(
				EPGSearch,
				evt.getEventName()
			)
		else:
			self.close(evt.getEventName())

