# -*- coding: utf-8 -*-
# GUI (Screens)
from Screens.Screen import Screen
from Components.ConfigList import ConfigListScreen

# GUI (Summary)
from Screens.Setup import SetupSummary

# GUI (Components)
from Components.ActionMap import ActionMap
from Components.Sources.StaticText import StaticText

# Configuration
from Components.config import config, getConfigListEntry, ConfigYesNo

class EPGSearchSetup(Screen, ConfigListScreen):
	skin = """<screen name="EPGSearchSetup" position="center,center" size="565,370">
		<ePixmap pixmap="skin_default/buttons/red.png" position="0,0" size="140,40" alphatest="on" />
		<ePixmap pixmap="skin_default/buttons/green.png" position="140,0" size="140,40" alphatest="on" />
		<widget source="key_red" render="Label" position="0,0" zPosition="1" size="140,40" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" />
		<widget source="key_green" render="Label" position="140,0" zPosition="1" size="140,40" font="Regular;20" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" />
		<widget name="config" position="5,50" size="555,250" scrollbarMode="showOnDemand" />
		<ePixmap pixmap="skin_default/div-h.png" position="0,301" zPosition="1" size="565,2" />
		<widget source="help" render="Label" position="5,305" size="555,63" font="Regular;21" />
	</screen>"""

	def __init__(self, session):
		Screen.__init__(self, session)



		# Summary
		self.setup_title = ("MyEPGSearch Setup")
		self.onChangedEntry = []

		ConfigListScreen.__init__(
			self,
			[
				getConfigListEntry(_("Length of History"), config.plugins.MyEpgSearch.history_length, _("How many entries to keep in the search history at most. 0 disables history entirely!")),
				getConfigListEntry(_("Save any search string in history"), config.plugins.MyEpgSearch.save_any_search_string , _("Save any search string in history (e.g. searches started via EPG) or only manually entered/edited search strings.")),
				getConfigListEntry(_("Add \"Search\" Button to EPG"), config.plugins.MyEpgSearch.add_search_to_epg , _("If this setting is enabled, the plugin adds a \"Search\" Button to the regular EPG.")),
				getConfigListEntry(_("Default search type"), config.plugins.MyEpgSearch.search_type , _("Configure EPG search type")),
				getConfigListEntry(_("Case sensitive search"), config.plugins.MyEpgSearch.search_casetype , _("Perform case sensitive or case insensitive EPG searches")),
				getConfigListEntry(_("Limit \"Search\" to Channels from Bouquet (TV or Radio)"), config.plugins.MyEpgSearch.limit_to_bouquet , _("If enabled, only search results for Channels within a Bouquet are shown (either TV or Radio Bouquets).")),
				getConfigListEntry(("Begrenzung auf aktuelles Bouquet (Favorit)"), config.plugins.MyEpgSearch.limit_to_active_bouquet ,("Nur in Sendern des aktuellen Bouquets suchen.")),
				getConfigListEntry(_("Type of match for search string"), config.plugins.MyEpgSearch.match_type , _("Choose how the search string should match the results.")),
				getConfigListEntry(_("Show event short description"), config.plugins.MyEpgSearch.show_short_description, _("Show the event short description in the result list. Useful when looking for a certain episode of a TV show.")),
				getConfigListEntry(_("Timeout for info pop up"), config.plugins.MyEpgSearch.show_popup_time, _("Duration for showing additional information for search filters")),
				getConfigListEntry(("Aktion bei OK"), config.plugins.MyEpgSearch.okAction, ("Aktion beim Drücken der OK-Taste")),
			],
			session = session,
			on_change = self.changed
		)
		def selectionChanged():
			if self["config"].current:
				self["config"].current[1].onDeselect(self.session)
			self["config"].current = self["config"].getCurrent()
			if self["config"].current:
				self["config"].current[1].onSelect(self.session)
			for x in self["config"].onSelectionChanged:
				x()
		self["config"].selectionChanged = selectionChanged
		self["config"].onSelectionChanged.append(self.updateHelp)

		# Initialize widgets
		self["key_green"] = StaticText(_("OK"))
		self["key_red"] = StaticText(_("Cancel"))
		self["help"] = StaticText()

		# Define Actions
		self["actions"] = ActionMap(["SetupActions"],
			{
				"cancel": self.keyCancel,
				"save": self.keySave,
			}
		)

		# Trigger change
		self.changed()

		self.onLayoutFinish.append(self.setCustomTitle)

	def setCustomTitle(self):
		self.setTitle(_("EPGSearch Setup"))

	def updateHelp(self):
		cur = self["config"].getCurrent()
		if cur:
			self["help"].text = cur[2]

	def changed(self):
		for x in self.onChangedEntry:
			x()

	def getCurrentEntry(self):
		return self["config"].getCurrent()[0]

	def getCurrentValue(self):
		return str(self["config"].getCurrent()[1].getText())

	def createSummary(self):
		return SetupSummary

