# -*- coding: utf-8 -*-
from Components.Language import language
from Tools.Directories import resolveFilename, SCOPE_PLUGINS, SCOPE_LANGUAGE
from os import environ as os_environ
import gettext

# Config
from Components.config import config, ConfigSet, ConfigSubsection, ConfigText, ConfigNumber, ConfigYesNo, ConfigSelection, ConfigSelectionNumber

def localeInit():
	lang = language.getLanguage()[:2] # getLanguage returns e.g. "fi_FI" for "language_country"
	os_environ["LANGUAGE"] = lang # Enigma doesn't set this (or LC_ALL, LC_MESSAGES, LANG). gettext needs it!
	gettext.bindtextdomain("EPGSearch", resolveFilename(SCOPE_PLUGINS, "Extensions/EPGSearch/locale"))

def _(txt):
	t = gettext.dgettext("EPGSearch", txt)
	if t == txt:
		t = gettext.gettext(txt)
	return t

localeInit()
language.addCallback(localeInit)

config.plugins.MyEpgSearch = ConfigSubsection()
config.plugins.MyEpgSearch.history = ConfigSet(choices = [])
config.plugins.MyEpgSearch.history_length = ConfigNumber(default = 10)
config.plugins.MyEpgSearch.add_search_to_epg = ConfigYesNo(default = True)
config.plugins.MyEpgSearch.search_type = ConfigSelection(default = "ask", choices = [
	("ask", _("Ask before search")),
	("title", _("Title")),
	("shortdesc", _("Short description")),
	("title_shortdesc", _("Title and short description")),
	("extdesc", _("Extended description")),
	("full", _("Title, short and extended description"))])
config.plugins.MyEpgSearch.search_casetype = ConfigYesNo(default = False)
config.plugins.MyEpgSearch.limit_to_bouquet = ConfigYesNo(default = False)
config.plugins.MyEpgSearch.limit_to_active_bouquet = ConfigYesNo(default = False)
config.plugins.MyEpgSearch.save_any_search_string = ConfigYesNo(default = False)
config.plugins.MyEpgSearch.show_short_description = ConfigYesNo(default = True)
config.plugins.MyEpgSearch.show_popup_time = ConfigSelectionNumber(default = 4, min = 0 , max = 15, stepwidth = 1, wraparound = True)
config.plugins.MyEpgSearch.okAction = ConfigSelection(default = "curCh", choices = [("detail",  ("Sendungsbeschreibung")), ("curCh",  ("Auf selektieren Sender umschalten")), ("curBack", ("Auf selektierten Sender umschalten / Zurück"))])

MATCH_ANY = "any"
MATCH_BEGIN = "begin"
MATCH_EXACT = "exact"
config.plugins.MyEpgSearch.match_type = ConfigSelection(default = MATCH_ANY, choices = [(MATCH_ANY, _("any substring")), (MATCH_BEGIN, _("beginning of a word")), (MATCH_EXACT, _("exact match"))])
